@@ -1391,6 +1391,24 @@ class EvalfMixin(object):
             verbose=<bool>
                 Print debug information (default=False)
 
+        Notes
+        =====
+
+        When Floats are naively substituted into an expression, precision errors
+        may adversely affect the result. For example, adding 1e16 (a Float) to 1
+        will truncate to 1e16; if 1e16 is then subtracted, the result will be 0.
+        That is exactly what happens in the following:
+
+        >>> from sympy.abc import x, y, z
+        >>> values = {x: 1e16, y: 1, z: 1e16}
+        >>> (x + y - z).subs(values)
+        0
+
+        Using the subs argument for evalf is the accurate way to evaluate such an
+        expression:
+
+        >>> (x + y - z).evalf(subs=values)
+        1.00000000000000
         """
         from sympy import Float, Number
         n = n if n is not None else 15
