@@ -94,6 +94,12 @@ class SingletonRegistry(Registry):
         # finished).
 
     def register(self, cls):
+        # Make sure a duplicate class overwrites the old one
+        try:
+            if getattr(self, cls.__name__):
+                delattr(self, cls.__name__)
+        except AttributeError:
+            pass
         self._classes_to_install[cls.__name__] = cls
 
     def __getattr__(self, name):
