@@ -1,27 +1,30 @@
 from __future__ import division
 
 from sympy.assumptions.ask import Q
+from sympy.core.containers import Tuple
 from sympy.core.numbers import oo
-from sympy.core.relational import Equality
+from sympy.core.relational import Equality, Eq, Ne
 from sympy.core.singleton import S
 from sympy.core.symbol import (Dummy, symbols)
+from sympy.functions import Piecewise
+from sympy.functions.elementary.trigonometric import sin
 from sympy.sets.sets import (EmptySet, Interval, Union)
 from sympy.simplify.simplify import simplify
 from sympy.logic.boolalg import (
     And, Boolean, Equivalent, ITE, Implies, Nand, Nor, Not, Or,
-    POSform, SOPform, Xor, conjuncts, disjuncts,
+    POSform, SOPform, Xor, Xnor, conjuncts, disjuncts,
     distribute_or_over_and, distribute_and_over_or,
     eliminate_implications, is_nnf, is_cnf, is_dnf, simplify_logic,
     to_nnf, to_cnf, to_dnf, to_int_repr, bool_map, true, false,
     BooleanAtom, is_literal, term_to_integer, integer_to_term,
-    truth_table)
+    truth_table, as_Boolean)
 
 from sympy.utilities.pytest import raises, XFAIL
 from sympy.utilities import cartes
 
 
-A, B, C, D= symbols('A,B,C,D')
-
+A, B, C, D = symbols('A:D')
+a, b, c, d, e, w, x, y, z = symbols('a:e w:z')
 
 def test_overloading():
     """Test that |, & are overloaded as expected"""
@@ -36,7 +39,6 @@ def test_overloading():
 
 
 def test_And():
-
     assert And() is true
     assert And(A) == A
     assert And(True) is true
@@ -49,8 +51,9 @@ def test_And():
     assert And(True, True, True) is true
     assert And(True, True, A) == A
     assert And(True, False, A) is false
-    assert And(2, A) == A
-    assert And(2, 3) is true
+    assert And(1, A) == A
+    raises(TypeError, lambda: And(2, A))
+    raises(TypeError, lambda: And(A < 2, A))
     assert And(A < 1, A >= 1) is false
     e = A > 1
     assert And(e, e.canonical) == e.canonical
@@ -59,7 +62,6 @@ def test_And():
 
 
 def test_Or():
-
     assert Or() is false
     assert Or(A) == A
     assert Or(True) is true
@@ -72,7 +74,9 @@ def test_Or():
     assert Or(True, False, False) is true
     assert Or(True, False, A) is true
     assert Or(False, False, A) == A
-    assert Or(2, A) is true
+    assert Or(1, A) is true
+    raises(TypeError, lambda: Or(2, A))
+    raises(TypeError, lambda: Or(A < 2, A))
     assert Or(A < 1, A >= 1) is true
     e = A > 1
     assert Or(e, e.canonical) == e
@@ -81,7 +85,6 @@ def test_Or():
 
 
 def test_Xor():
-
     assert Xor() is false
     assert Xor(A) == A
     assert Xor(A, A) is false
@@ -107,7 +110,6 @@ def test_Xor():
 
 
 def test_Not():
-
     raises(TypeError, lambda: Not(True, False))
     assert Not(True) is false
     assert Not(False) is true
@@ -117,7 +119,6 @@ def test_Not():
 
 
 def test_Nand():
-
     assert Nand() is false
     assert Nand(A) == ~A
     assert Nand(True) is false
@@ -133,7 +134,6 @@ def test_Nand():
 
 
 def test_Nor():
-
     assert Nor() is true
     assert Nor(A) == ~A
     assert Nor(True) is false
@@ -147,9 +147,25 @@ def test_Nor():
     assert Nor(True, True, A) is false
     assert Nor(True, False, A) is false
 
+def test_Xnor():
+    assert Xnor() is true
+    assert Xnor(A) == ~A
+    assert Xnor(A, A) is true
+    assert Xnor(True, A, A) is false
+    assert Xnor(A, A, A, A, A) == ~A
+    assert Xnor(True) is false
+    assert Xnor(False) is true
+    assert Xnor(True, True ) is true
+    assert Xnor(True, False) is false
+    assert Xnor(False, False) is true
+    assert Xnor(True, A) == A
+    assert Xnor(False, A) == ~A
+    assert Xnor(True, False, False) is false
+    assert Xnor(True, False, A) == A
+    assert Xnor(False, False, A) == ~A
+
 
 def test_Implies():
-
     raises(ValueError, lambda: Implies(A, B, C))
     assert Implies(True, True) is true
     assert Implies(True, False) is false
@@ -165,7 +181,6 @@ def test_Implies():
 
 
 def test_Equivalent():
-
     assert Equivalent(A, B) == Equivalent(B, A) == Equivalent(A, B, A)
     assert Equivalent() is true
     assert Equivalent(A, A) == Equivalent(A) is true
@@ -191,7 +206,6 @@ def test_equals():
     assert ((A | ~B) & (~A | B)).equals((~A & ~B) | (A & B)) is True
     assert (A >> B).equals(~A >> ~B) is False
     assert (A >> (B >> A)).equals(A >> (C >> A)) is False
-    raises(NotImplementedError, lambda: And(A, A < B).equals(And(A, B > A)))
 
 
 def test_simplification():
@@ -200,7 +214,6 @@ def test_simplification():
     """
     set1 = [[0, 0, 1], [0, 1, 1], [1, 0, 0], [1, 1, 0]]
     set2 = [[0, 0, 0], [0, 1, 0], [1, 0, 1], [1, 1, 1]]
-    from sympy.abc import w, x, y, z
     assert SOPform([x, y, z], set1) == Or(And(Not(x), z), And(Not(z), x))
     assert Not(SOPform([x, y, z], set2)) == Not(Or(And(Not(x), Not(z)), And(x, z)))
     assert POSform([x, y, z], set1 + set2) is true
@@ -223,13 +236,14 @@ def test_simplification():
     assert simplify_logic(Equivalent(A, B)) == \
            Or(And(A, B), And(Not(A), Not(B)))
     assert simplify_logic(And(Equality(A, 2), C)) == And(Equality(A, 2), C)
+    assert simplify_logic(And(Equality(A, 2), A)) is S.false
     assert simplify_logic(And(Equality(A, 2), A)) == And(Equality(A, 2), A)
     assert simplify_logic(And(Equality(A, B), C)) == And(Equality(A, B), C)
     assert simplify_logic(Or(And(Equality(A, 3), B), And(Equality(A, 3), C))) \
            == And(Equality(A, 3), Or(B, C))
-    e = And(A, x**2 - x)
-    assert simplify_logic(e) == And(A, x*(x - 1))
-    assert simplify_logic(e, deep=False) == e
+    b = (~x & ~y & ~z) | ( ~x & ~y & z)
+    e = And(A, b)
+    assert simplify_logic(e) == A & ~x & ~y
 
     # check input
     ans = SOPform([x, y], [[1, 0]])
@@ -259,7 +273,6 @@ def test_bool_map():
 
     minterms = [[0, 0, 0, 1], [0, 0, 1, 1], [0, 1, 1, 1], [1, 0, 1, 1],
         [1, 1, 1, 1]]
-    from sympy.abc import a, b, c, w, x, y, z
     assert bool_map(Not(Not(a)), a) == (a, {a: a})
     assert bool_map(SOPform([w, x, y, z], minterms),
         POSform([w, x, y, z], minterms)) == \
@@ -285,7 +298,6 @@ def test_bool_symbol():
 
 
 def test_is_boolean():
-
     assert true.is_Boolean
     assert (A & B).is_Boolean
     assert (A | B).is_Boolean
@@ -294,7 +306,6 @@ def test_is_boolean():
 
 
 def test_subs():
-
     assert (A & B).subs(A, True) == B
     assert (A & B).subs(A, False) is false
     assert (A & B).subs(B, True) == A
@@ -327,7 +338,6 @@ def test_and_associativity():
 
 
 def test_or_assicativity():
-
     assert ((A | B) | C) == (A | (B | C))
 
 
@@ -339,7 +349,6 @@ def test_double_negation():
 # test methods
 
 def test_eliminate_implications():
-    from sympy.abc import A, B, C, D
     assert eliminate_implications(Implies(A, B, evaluate=False)) == (~A) | B
     assert eliminate_implications(
         A >> (C >> Not(B))) == Or(Or(Not(B), Not(C)), Not(A))
@@ -348,23 +357,22 @@ def test_eliminate_implications():
 
 
 def test_conjuncts():
-    assert conjuncts(A & B & C) == set([A, B, C])
-    assert conjuncts((A | B) & C) == set([A | B, C])
-    assert conjuncts(A) == set([A])
-    assert conjuncts(True) == set([True])
-    assert conjuncts(False) == set([False])
+    assert conjuncts(A & B & C) == {A, B, C}
+    assert conjuncts((A | B) & C) == {A | B, C}
+    assert conjuncts(A) == {A}
+    assert conjuncts(True) == {True}
+    assert conjuncts(False) == {False}
 
 
 def test_disjuncts():
-    assert disjuncts(A | B | C) == set([A, B, C])
-    assert disjuncts((A | B) & C) == set([(A | B) & C])
-    assert disjuncts(A) == set([A])
-    assert disjuncts(True) == set([True])
-    assert disjuncts(False) == set([False])
+    assert disjuncts(A | B | C) == {A, B, C}
+    assert disjuncts((A | B) & C) == {(A | B) & C}
+    assert disjuncts(A) == {A}
+    assert disjuncts(True) == {True}
+    assert disjuncts(False) == {False}
 
 
 def test_distribute():
-
     assert distribute_and_over_or(Or(And(A, B), C)) == And(Or(A, C), Or(B, C))
     assert distribute_or_over_and(And(A, Or(B, C))) == Or(And(A, B), And(A, C))
 
@@ -390,10 +398,14 @@ def test_to_nnf():
     assert to_nnf((A >> B) ^ (B >> A)) == (A & ~B) | (~A & B)
     assert to_nnf((A >> B) ^ (B >> A), False) == \
             (~A | ~B | A | B) & ((A & ~B) | (~A & B))
+    assert ITE(A, 1, 0).to_nnf() == A
+    assert ITE(A, 0, 1).to_nnf() == ~A
+    # although ITE can hold non-Boolean, it will complain if
+    # an attempt is made to convert the ITE to Boolean nnf
+    raises(TypeError, lambda: ITE(A < 1, [1], B).to_nnf())
 
 
 def test_to_cnf():
-
     assert to_cnf(~(B | C)) == And(Not(B), Not(C))
     assert to_cnf((A & B) | C) == And(Or(A, C), Or(B, C))
     assert to_cnf(A >> B) == (~A) | B
@@ -408,7 +420,6 @@ def test_to_cnf():
 
 
 def test_to_dnf():
-
     assert to_dnf(~(B | C)) == And(Not(B), Not(C))
     assert to_dnf(A & (B | C)) == Or(And(A, B), And(A, C))
     assert to_dnf(A >> B) == (~A) | B
@@ -436,7 +447,6 @@ def test_to_int_repr():
 
 
 def test_is_nnf():
-    from sympy.abc import A, B
     assert is_nnf(true) is True
     assert is_nnf(A) is True
     assert is_nnf(~A) is True
@@ -449,7 +459,6 @@ def test_is_nnf():
 
 
 def test_is_cnf():
-    x, y, z = symbols('x,y,z')
     assert is_cnf(x) is True
     assert is_cnf(x | y | z) is True
     assert is_cnf(x & y & z) is True
@@ -458,7 +467,6 @@ def test_is_cnf():
 
 
 def test_is_dnf():
-    x, y, z = symbols('x,y,z')
     assert is_dnf(x) is True
     assert is_dnf(x | y | z) is True
     assert is_dnf(x & y & z) is True
@@ -467,8 +475,7 @@ def test_is_dnf():
 
 
 def test_ITE():
-    A, B, C = map(Boolean, symbols('A,B,C'))
-
+    A, B, C = symbols('A:C')
     assert ITE(True, False, True) is false
     assert ITE(True, True, False) is true
     assert ITE(False, True, False) is false
@@ -482,12 +489,31 @@ def test_ITE():
     B = True
     assert ITE(And(A, B), B, C) == C
     assert ITE(Or(A, False), And(B, True), False) is false
+    assert ITE(x, A, B) == Not(x)
+    assert ITE(x, B, A) == x
+    assert ITE(1, x, y) == x
+    assert ITE(0, x, y) == y
+    raises(TypeError, lambda: ITE(2, x, y))
+    raises(TypeError, lambda: ITE(1, [], y))
+    raises(TypeError, lambda: ITE(1, (), y))
+    raises(TypeError, lambda: ITE(1, y, []))
+    assert ITE(1, 1, 1) is S.true
+    assert isinstance(ITE(1, 1, 1, evaluate=False), ITE)
 
-
-def test_ITE_diff():
-    # analogous to Piecewise.diff
-    x = symbols('x')
-    assert ITE(x > 0, x**2, x).diff(x) == ITE(x > 0, 2*x, 1)
+    raises(TypeError, lambda: ITE(x > 1, y, x))
+    assert ITE(Eq(x, True), y, x) == ITE(x, y, x)
+    assert ITE(Eq(x, False), y, x) == ITE(~x, y, x)
+    assert ITE(Ne(x, True), y, x) == ITE(~x, y, x)
+    assert ITE(Ne(x, False), y, x) == ITE(x, y, x)
+    # 0 and 1 in the context are not treated as True/False
+    # so the equality must always be False since dissimilar
+    # objects cannot be equal
+    assert ITE(Eq(x, 0), y, x) == x
+    assert ITE(Eq(x, 1), y, x) == x
+    assert ITE(Ne(x, 0), y, x) == y
+    assert ITE(Ne(x, 1), y, x) == y
+    assert ITE(Eq(x, 0), y, z).subs(x, 0) == y
+    assert ITE(Eq(x, 0), y, z).subs(x, 1) == z
 
 
 def test_is_literal():
@@ -522,8 +548,6 @@ def test_operators():
 
 
 def test_true_false():
-    x = symbols('x')
-
     assert true is S.true
     assert false is S.false
     assert true is not True
@@ -538,7 +562,7 @@ def test_true_false():
 
     assert hash(true) == hash(True)
     assert hash(false) == hash(False)
-    assert len(set([true, True])) == len(set([false, False])) == 1
+    assert len({true, True}) == len({false, False}) == 1
 
     assert isinstance(true, BooleanAtom)
     assert isinstance(false, BooleanAtom)
@@ -654,10 +678,11 @@ def test_true_false():
         assert ITE(F, F, T) is true
         assert ITE(F, F, F) is false
 
+    assert all(i.simplify(1, 2) is i for i in (S.true, S.false))
+
 
 def test_bool_as_set():
-    x = symbols('x')
-
+    assert ITE(y <= 0, False, y >= 1).as_set() == Interval(1, oo)
     assert And(x <= 2, x >= -2).as_set() == Interval(-2, 2)
     assert Or(x >= 2, x <= -2).as_set() == Interval(-oo, -2) + Interval(2, oo)
     assert Not(x > 2).as_set() == Interval(-oo, 2)
@@ -666,6 +691,11 @@ def test_bool_as_set():
         Union(Interval(-oo,2),Interval(3,oo))
     assert true.as_set() == S.UniversalSet
     assert false.as_set() == EmptySet()
+    assert x.as_set() == S.UniversalSet
+    assert And(Or(x < 1, x > 3), x < 2
+        ).as_set() == Interval.open(-oo, 1)
+    assert And(x < 1, sin(x) < 3).as_set() == (x < 1).as_set()
+    raises(NotImplementedError, lambda: (sin(x) < 1).as_set())
 
 
 @XFAIL
@@ -698,7 +728,6 @@ def test_canonical_atoms():
 
 
 def test_issue_8777():
-    x = symbols('x')
     assert And(x > 2, x < oo).as_set() == Interval(2, oo, left_open=True)
     assert And(x >= 1, x < oo).as_set() == Interval(1, oo)
     assert (x < oo).as_set() == Interval(-oo, oo)
@@ -706,7 +735,6 @@ def test_issue_8777():
 
 
 def test_issue_8975():
-    x = symbols('x')
     assert Or(And(-oo < x, x <= -2), And(2 <= x, x < oo)).as_set() == \
         Interval(-oo, -2) + Interval(2, oo)
 
@@ -723,14 +751,12 @@ def test_integer_to_term():
 
 
 def test_truth_table():
-    x, y = symbols('x,y')
     assert list(truth_table(And(x, y), [x, y], input=False)) == [False, False, False, True]
     assert list(truth_table(x | y, [x, y], input=False)) == [False, True, True, True]
     assert list(truth_table(x >> y, [x, y], input=False)) == [True, True, False, True]
 
 
 def test_issue_8571():
-    x = symbols('x')
     for t in (S.true, S.false):
         raises(TypeError, lambda: +t)
         raises(TypeError, lambda: -t)
@@ -747,3 +773,48 @@ def test_issue_8571():
                 raises(TypeError, lambda: o/t)
                 raises(TypeError, lambda: o**t)
                 o, t = t, o  # do again in reversed order
+
+
+def test_expand_relational():
+    n = symbols('n', negative=True)
+    p, q = symbols('p q', positive=True)
+    r = ((n + q*(-n/q + 1))/(q*(-n/q + 1)) < 0)
+    assert r is not S.false
+    assert r.expand() is S.false
+    assert (q > 0).expand() is S.true
+
+
+def test_issue_12717():
+    assert S.true.is_Atom == True
+    assert S.false.is_Atom == True
+
+
+def test_as_Boolean():
+    nz = symbols('nz', nonzero=True)
+    assert all(as_Boolean(i) is S.true for i in (True, S.true, 1, nz))
+    z = symbols('z', zero=True)
+    assert all(as_Boolean(i) is S.false for i in (False, S.false, 0, z))
+    assert all(as_Boolean(i) == i for i in (x, x < 0))
+    for i in (2, S(2), x + 1, []):
+        raises(TypeError, lambda: as_Boolean(i))
+
+
+def test_binary_symbols():
+    assert ITE(x < 1, y, z).binary_symbols == set((y, z))
+    for f in (Eq, Ne):
+        assert f(x, 1).binary_symbols == set()
+        assert f(x, True).binary_symbols == set([x])
+        assert f(x, False).binary_symbols == set([x])
+    assert S.true.binary_symbols == set()
+    assert S.false.binary_symbols == set()
+    assert x.binary_symbols == set([x])
+    assert And(x, Eq(y, False), Eq(z, 1)).binary_symbols == set([x, y])
+    assert Q.prime(x).binary_symbols == set()
+    assert Q.is_true(x < 1).binary_symbols == set()
+    assert Q.is_true(x).binary_symbols == set([x])
+    assert Q.is_true(Eq(x, True)).binary_symbols == set([x])
+    assert Q.prime(x).binary_symbols == set()
+
+
+def test_BooleanFunction_diff():
+    assert And(x, y).diff(x) == Piecewise((0, Eq(False, y)), (1, True))
