@@ -13,6 +13,8 @@ def test_Vector():
     assert A.y != A.z
     assert A.z != A.x
 
+    assert A.x + 0 == A.x
+
     v1 = x*A.x + y*A.y + z*A.z
     v2 = x**2*A.x + y**2*A.y + z**2*A.z
     v3 = v1 + v2
@@ -54,6 +56,10 @@ def test_Vector():
     assert v1.separate() == {A: v1}
     assert v5.separate() == {A: x*A.x + y*A.y, B: z*B.z}
 
+    #Test the free_symbols property
+    v6 = x*A.x + y*A.y + z*A.z
+    assert v6.free_symbols(A) == {x,y,z}
+
 
 def test_Vector_diffs():
     q1, q2, q3, q4 = dynamicsymbols('q1 q2 q3 q4')
