@@ -248,7 +248,7 @@ class PythonRational(DefaultPrinting, PicklableWithSlots, DomainElement):
             return False
 
     def __ne__(self, other):
-        return not self.__eq__(other)
+        return not self == other
 
     def _cmp(self, other, op):
         try:
