@@ -53,7 +53,8 @@ class CodePrinter(StrPrinter):
         'error_on_reserved': False,
         'reserved_word_suffix': '_',
         'human': True,
-        'inline': False
+        'inline': False,
+        'allow_unknown_functions': False,
     }
 
     def __init__(self, settings=None):
@@ -332,7 +333,7 @@ class CodePrinter(StrPrinter):
         rhs_code = self._print(expr.rhs)
         return self._get_statement("{0} {1} {2}".format(
             *map(lambda arg: self._print(arg),
-                 [lhs_code, expr.rel_op, rhs_code])))
+                 [lhs_code, expr.op, rhs_code])))
 
     def _print_FunctionCall(self, expr):
         return '%s(%s)' % (
@@ -381,6 +382,8 @@ class CodePrinter(StrPrinter):
         elif hasattr(expr, '_imp_') and isinstance(expr._imp_, Lambda):
             # inlined function
             return self._print(expr._imp_(*expr.args))
+        elif expr.is_Function and self._settings.get('allow_unknown_functions', False):
+            return '%s(%s)' % (self._print(expr.func), ', '.join(map(self._print, expr.args)))
         else:
             return self._print_not_supported(expr)
 
@@ -402,6 +405,8 @@ class CodePrinter(StrPrinter):
         return self._print_NumberSymbol(expr)
     def _print_GoldenRatio(self, expr):
         return self._print_NumberSymbol(expr)
+    def _print_TribonacciConstant(self, expr):
+        return self._print_NumberSymbol(expr)
     def _print_Exp1(self, expr):
         return self._print_NumberSymbol(expr)
     def _print_Pi(self, expr):
