@@ -129,10 +129,10 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
         (a + b)*Derivative(f(x), x)
 
         >>> collect(a*D(D(f,x),x) + b*D(D(f,x),x), f)
-        (a + b)*Derivative(f(x), x, x)
+        (a + b)*Derivative(f(x), (x, 2))
 
         >>> collect(a*D(D(f,x),x) + b*D(D(f,x),x), D(f,x), exact=True)
-        a*Derivative(f(x), x, x) + b*Derivative(f(x), x, x)
+        a*Derivative(f(x), (x, 2)) + b*Derivative(f(x), (x, 2))
 
         >>> collect(a*D(f,x) + b*D(f,x) + a*f + b*f, f)
         (a + b)*f(x) + (a + b)*Derivative(f(x), x)
@@ -140,7 +140,7 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
     Or you can even match both derivative order and exponent at the same time::
 
         >>> collect(a*D(D(f,x),x)**2 + b*D(D(f,x),x)**2, D(f,x))
-        (a + b)*Derivative(f(x), x, x)**2
+        (a + b)*Derivative(f(x), (x, 2))**2
 
     Finally, you can apply a function to each of the collected coefficients.
     For example you can factorize symbolic coefficients of polynomial::
@@ -157,6 +157,9 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
     ========
     collect_const, collect_sqrt, rcollect
     """
+    expr = sympify(expr)
+    syms = list(syms) if iterable(syms) else [syms]
+
     if evaluate is None:
         evaluate = global_evaluate[0]
 
@@ -237,7 +240,7 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
                     rat_expo, sym_expo = coeff, tail
                 else:
                     sym_expo = expr.exp
-        elif expr.func is exp:
+        elif isinstance(expr, exp):
             arg = expr.args[0]
             if arg.is_Rational:
                 sexpr, rat_expo = S.Exp1, arg
@@ -331,12 +334,8 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
                 expr.base, syms, func, True, exact, distribute_order_term)
             return Pow(b, expr.exp)
 
-    if iterable(syms):
-        syms = [expand_power_base(i, deep=False) for i in syms]
-    else:
-        syms = [expand_power_base(syms, deep=False)]
+    syms = [expand_power_base(i, deep=False) for i in syms]
 
-    expr = sympify(expr)
     order_term = None
 
     if distribute_order_term:
@@ -352,7 +351,10 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
 
     collected, disliked = defaultdict(list), S.Zero
     for product in summa:
-        terms = [parse_term(i) for i in Mul.make_args(product)]
+        c, nc = product.args_cnc(split_1=False)
+        args = list(ordered(c)) + nc
+        terms = [parse_term(i) for i in args]
+        small_first = True
 
         for symbol in syms:
             if SYMPY_DEBUG:
@@ -360,6 +362,9 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
                     str(terms), str(symbol))
                 )
 
+            if isinstance(symbol, Derivative) and small_first:
+                terms = list(reversed(terms))
+                small_first = not small_first
             result = parse_expression(terms, symbol)
 
             if SYMPY_DEBUG:
@@ -371,12 +376,14 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
                 # when there was derivative in current pattern we
                 # will need to rebuild its expression from scratch
                 if not has_deriv:
-                    index = 1
+                    margs = []
                     for elem in elems:
-                        e = elem[1]
-                        if elem[2] is not None:
-                            e *= elem[2]
-                        index *= Pow(elem[0], e)
+                        if elem[2] is None:
+                            e = elem[1]
+                        else:
+                            e = elem[1]*elem[2]
+                        margs.append(Pow(elem[0], e))
+                    index = Mul(*margs)
                 else:
                     index = make_expression(elems)
                 terms = expand_power_base(make_expression(terms), deep=False)
@@ -387,7 +394,7 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
             # none of the patterns matched
             disliked += product
     # add terms now for each key
-    collected = dict([(k, Add(*v)) for k, v in collected.items()])
+    collected = {k: Add(*v) for k, v in collected.items()}
 
     if disliked is not S.Zero:
         collected[S.One] = disliked
@@ -520,6 +527,29 @@ def collect_const(expr, *vars, **kwargs):
     targeted. Although any Number can also be targeted, if this is not
     desired set ``Numbers=False`` and no Float or Rational will be collected.
 
+    Parameters
+    ==========
+
+    expr : sympy expression
+        This parameter defines the expression the expression from which
+        terms with similar coefficients are to be collected. A non-Add
+        expression is returned as it is.
+
+    vars : variable length collection of Numbers, optional
+        Specifies the constants to target for collection. Can be multiple in
+        number.
+
+    kwargs : ``Numbers`` is the only possible argument to pass.
+        Numbers (default=True) specifies to target all instance of
+        :class:`sympy.core.numbers.Number` class. If ``Numbers=False``, then
+        no Float or Rational will be collected.
+
+    Returns
+    =======
+
+    expr : Expr
+        Returns an expression with similar coefficient terms collected.
+
     Examples
     ========
 
@@ -628,7 +658,7 @@ def collect_const(expr, *vars, **kwargs):
 
 
 def radsimp(expr, symbolic=True, max_terms=4):
-    """
+    r"""
     Rationalize the denominator by removing square roots.
 
     Note: the expression returned from radsimp must be used with caution
@@ -651,8 +681,6 @@ def radsimp(expr, symbolic=True, max_terms=4):
     >>> from sympy.simplify.radsimp import collect_sqrt
     >>> from sympy.abc import a, b, c
 
-    >>> radsimp(1/(I + 1))
-    (1 - I)/2
     >>> radsimp(1/(2 + sqrt(2)))
     (-sqrt(2) + 2)/2
     >>> x,y = map(Symbol, 'xy')
@@ -735,14 +763,14 @@ def radsimp(expr, symbolic=True, max_terms=4):
         if not d.is_Pow:
             return False
         e = d.exp
-        if e.is_Rational and e.q == 2 or symbolic and fraction(e)[1] == 2:
+        if e.is_Rational and e.q == 2 or symbolic and denom(e) == 2:
             return True
         if log2:
             q = 1
             if e.is_Rational:
                 q = e.q
             elif symbolic:
-                d = fraction(e)[1]
+                d = denom(e)
                 if d.is_Integer:
                     q = d
             if q != 1 and log(q, 2).is_Integer:
@@ -772,7 +800,7 @@ def radsimp(expr, symbolic=True, max_terms=4):
             return expr
 
         if ispow2(d):
-            d2 = sqrtdenest(sqrt(d.base))**fraction(d.exp)[0]
+            d2 = sqrtdenest(sqrt(d.base))**numer(d.exp)
             if d2 != d:
                 return handle(1/d2)
         elif d.is_Pow and (d.exp.is_integer or d.base.is_positive):
@@ -936,7 +964,7 @@ def fraction(expr, exact=False):
        flag is unset, then structure this exponent's structure will
        be analyzed and pretty fraction will be returned:
 
-       >>> from sympy import exp
+       >>> from sympy import exp, Mul
        >>> fraction(2*x**(-y))
        (2, x**y)
 
@@ -946,17 +974,30 @@ def fraction(expr, exact=False):
        >>> fraction(exp(-x), exact=True)
        (exp(-x), 1)
 
+       The `exact` flag will also keep any unevaluated Muls from
+       being evaluated:
+
+       >>> u = Mul(2, x + 1, evaluate=False)
+       >>> fraction(u)
+       (2*x + 2, 1)
+       >>> fraction(u, exact=True)
+       (2*(x  + 1), 1)
     """
     expr = sympify(expr)
 
     numer, denom = [], []
 
     for term in Mul.make_args(expr):
-        if term.is_commutative and (term.is_Pow or term.func is exp):
+        if term.is_commutative and (term.is_Pow or isinstance(term, exp)):
             b, ex = term.as_base_exp()
             if ex.is_negative:
                 if ex is S.NegativeOne:
                     denom.append(b)
+                elif exact:
+                    if ex.is_constant():
+                        denom.append(Pow(b, -ex))
+                    else:
+                        numer.append(term)
                 else:
                     denom.append(Pow(b, -ex))
             elif ex.is_positive:
@@ -973,8 +1014,10 @@ def fraction(expr, exact=False):
             denom.append(d)
         else:
             numer.append(term)
-
-    return Mul(*numer), Mul(*denom)
+    if exact:
+        return Mul(*numer, evaluate=False), Mul(*denom, evaluate=False)
+    else:
+        return Mul(*numer), Mul(*denom)
 
 
 def numer(expr):
