@@ -47,14 +47,17 @@ from __future__ import print_function, division
 
 from sympy import (log, sqrt, pi, S, Dummy, Interval, sympify, gamma,
                    Piecewise, And, Eq, binomial, factorial, Sum, floor, Abs,
-                   Lambda, Basic, lowergamma, erf, erfi, erfc, I, hyper, uppergamma,
-                   sinh, Ne)
+                   Lambda, Basic, lowergamma, erf, erfi, I, hyper, uppergamma,
+                   sinh, Ne, expint)
 
 from sympy import beta as beta_fn
-from sympy import cos, sin, exp, besseli, besselj
+from sympy import cos, sin, exp, besseli, besselj, besselk
 from sympy.stats.crv import (SingleContinuousPSpace, SingleContinuousDistribution,
         ContinuousDistributionHandmade)
-from sympy.stats.rv import _value_check
+from sympy.stats.rv import _value_check, RandomSymbol
+from sympy.matrices import MatrixBase
+from sympy.stats.joint_rv_types import multivariate_rv
+from sympy.stats.joint_rv import MarginalDistribution, JointPSpace, CompoundDistribution
 from sympy.external import import_module
 import random
 
@@ -132,7 +135,8 @@ def ContinuousRV(symbol, density, set=Interval(-oo, oo)):
     >>> P(X>0)
     1/2
     """
-    pdf = Lambda(symbol, density)
+    pdf = Piecewise((density, set.as_relational(symbol)), (0, True))
+    pdf = Lambda(symbol, pdf)
     dist = ContinuousDistributionHandmade(pdf, set)
     return SingleContinuousPSpace(symbol, dist).value
 
@@ -141,7 +145,10 @@ def rv(symbol, cls, args):
     args = list(map(sympify, args))
     dist = cls(*args)
     dist.check(*args)
-    return SingleContinuousPSpace(symbol, dist).value
+    pspace = SingleContinuousPSpace(symbol, dist)
+    if any(isinstance(arg, RandomSymbol) for arg in args):
+        pspace = JointPSpace(symbol, CompoundDistribution(dist))
+    return pspace.value
 
 ########################################
 # Continuous Probability Distributions #
@@ -233,6 +240,9 @@ class BeniniDistribution(SingleContinuousDistribution):
         return (exp(-alpha*log(x/sigma) - beta*log(x/sigma)**2)
                *(alpha/x + 2*beta*log(x/sigma)/x))
 
+    def _moment_generating_function(self, t):
+        raise NotImplementedError('The moment generating function of the '
+                                  'Benini distribution does not exist.')
 
 def Benini(name, alpha, beta, sigma):
     r"""
@@ -314,6 +324,8 @@ class BetaDistribution(SingleContinuousDistribution):
     def _characteristic_function(self, t):
         return hyper((self.alpha,), (self.alpha + self.beta,), I*t)
 
+    def _moment_generating_function(self, t):
+        return hyper((self.alpha,), (self.alpha + self.beta,), t)
 
 def Beta(name, alpha, beta):
     r"""
@@ -384,7 +396,6 @@ class BetaPrimeDistribution(SingleContinuousDistribution):
         alpha, beta = self.alpha, self.beta
         return x**(alpha - 1)*(1 + x)**(-alpha - beta)/beta_fn(alpha, beta)
 
-
 def BetaPrime(name, alpha, beta):
     r"""
     Create a continuous random variable with a Beta prime distribution.
@@ -445,6 +456,12 @@ class CauchyDistribution(SingleContinuousDistribution):
     def pdf(self, x):
         return 1/(pi*self.gamma*(1 + ((x - self.x0)/self.gamma)**2))
 
+    def _characteristic_function(self, t):
+        return exp(self.x0 * I * t -  self.gamma * Abs(t))
+
+    def _moment_generating_function(self, t):
+        raise NotImplementedError("The moment generating function for the "
+                                  "Cauchy distribution does not exist.")
 
 def Cauchy(name, x0, gamma):
     r"""
@@ -511,6 +528,14 @@ class ChiDistribution(SingleContinuousDistribution):
         part_3 = hyper(((k+1)/2,), (S(3)/2,), -t**2/2)
         return part_1 + part_2*part_3
 
+    def _moment_generating_function(self, t):
+        k = self.k
+
+        part_1 = hyper((k / 2,), (S(1) / 2,), t ** 2 / 2)
+        part_2 = t * sqrt(2) * gamma((k + 1) / 2) / gamma(k / 2)
+        part_3 = hyper(((k + 1) / 2,), (S(3) / 2,), t ** 2 / 2)
+        return part_1 + part_2 * part_3
+
 def Chi(name, k):
     r"""
     Create a continuous random variable with a Chi distribution.
@@ -568,7 +593,6 @@ class ChiNoncentralDistribution(SingleContinuousDistribution):
         k, l = self.k, self.l
         return exp(-(x**2+l**2)/2)*x**k*l / (l*x)**(k/2) * besseli(k/2-1, l*x)
 
-
 def ChiNoncentral(name, k, l):
     r"""
     Create a continuous random variable with a non-central Chi distribution.
@@ -639,6 +663,9 @@ class ChiSquaredDistribution(SingleContinuousDistribution):
     def _characteristic_function(self, t):
         return (1 - 2*I*t)**(-self.k/2)
 
+    def  _moment_generating_function(self, t):
+        return (1 - 2*t)**(-self.k/2)
+
 def ChiSquared(name, k):
     r"""
     Create a continuous random variable with a Chi-squared distribution.
@@ -706,7 +733,6 @@ class DagumDistribution(SingleContinuousDistribution):
         return Piecewise(((S.One + (S(x)/b)**-a)**-p, x>=0),
                     (S.Zero, True))
 
-
 def Dagum(name, p, a, b):
     r"""
     Create a continuous random variable with a Dagum distribution.
@@ -858,6 +884,10 @@ class ExponentialDistribution(SingleContinuousDistribution):
         rate = self.rate
         return rate / (rate - I*t)
 
+    def _moment_generating_function(self, t):
+        rate = self.rate
+        return rate / (rate - t)
+
 def Exponential(name, rate):
     r"""
     Create a continuous random variable with an Exponential distribution.
@@ -940,6 +970,9 @@ class FDistributionDistribution(SingleContinuousDistribution):
         return (sqrt((d1*x)**d1*d2**d2 / (d1*x+d2)**(d1+d2))
                / (x * beta_fn(d1/2, d2/2)))
 
+    def _moment_generating_function(self, t):
+        raise NotImplementedError('The moment generating function for the '
+                                  'F-distribution does not exist.')
 
 def FDistribution(name, d1, d2):
     r"""
@@ -1164,6 +1197,11 @@ class GammaDistribution(SingleContinuousDistribution):
                     (lowergamma(k, S(x)/theta)/gamma(k), x > 0),
                     (S.Zero, True))
 
+    def _characteristic_function(self, t):
+        return (1 - self.theta*I*t)**(-self.k)
+
+    def _moment_generating_function(self, t):
+        return (1- self.theta*t)**(-self.k)
 
 def Gamma(name, k, theta):
     r"""
@@ -1219,7 +1257,7 @@ def Gamma(name, k, theta):
     \          0             otherwise
 
     >>> E(X)
-    theta*gamma(k + 1)/gamma(k)
+    k*theta
 
     >>> V = simplify(variance(X))
     >>> pprint(V, use_unicode=False)
@@ -1267,6 +1305,14 @@ class GammaInverseDistribution(SingleContinuousDistribution):
         else:
             raise NotImplementedError('Sampling the inverse Gamma Distribution requires Scipy.')
 
+    def _characteristic_function(self, t):
+        a, b = self.a, self.b
+        return 2 * (-I*b*t)**(a/2) * besselk(sqrt(-4*I*b*t)) / gamma(a)
+
+    def _moment_generating_function(self, t):
+        raise NotImplementedError('The moment generating function for the '
+                                  'gamma inverse distribution does not exist.')
+
 def GammaInverse(name, a, b):
     r"""
     Create a continuous random variable with an inverse Gamma distribution.
@@ -1336,6 +1382,12 @@ class GumbelDistribution(SingleContinuousDistribution):
         beta, mu = self.beta, self.mu
         return (1/beta)*exp(-((x-mu)/beta)+exp(-((x-mu)/beta)))
 
+    def _characteristic_function(self, t):
+        return gamma(1 - I*self.beta*t) * exp(I*self.mu*t)
+
+    def _moment_generating_function(self, t):
+        return gamma(1 - self.beta*t) * exp(I*self.mu*t)
+
 def Gumbel(name, beta, mu):
     r"""
     Create a Continuous Random Variable with Gumbel distribution.
@@ -1395,6 +1447,9 @@ class GompertzDistribution(SingleContinuousDistribution):
         eta, b = self.eta, self.b
         return b*eta*exp(b*x)*exp(eta)*exp(-eta*exp(b*x))
 
+    def _moment_generating_function(self, t):
+        eta, b = self.eta, self.b
+        return eta * exp(eta) * expint(t/b, eta)
 
 def Gompertz(name, b, eta):
     r"""
@@ -1466,7 +1521,6 @@ class KumaraswamyDistribution(SingleContinuousDistribution):
             (1 - (1 - x**a)**b, x <= S.One),
             (S.One, True))
 
-
 def Kumaraswamy(name, a, b):
     r"""
     Create a Continuous Random Variable with a Kumaraswamy distribution.
@@ -1538,6 +1592,11 @@ class LaplaceDistribution(SingleContinuousDistribution):
                     (S.One - S.Half*exp(-(x - mu)/b), x >= mu)
                         )
 
+    def _characteristic_function(self, t):
+        return exp(self.mu*I*t) / (1 + self.b**2*t**2)
+
+    def _moment_generating_function(self, t):
+        return exp(self.mu*t) / (1 - self.b**2*t**2)
 
 def Laplace(name, mu, b):
     r"""
@@ -1551,8 +1610,10 @@ def Laplace(name, mu, b):
     Parameters
     ==========
 
-    mu : Real number, the location (mean)
-    b : Real number, `b > 0`, a scale
+    mu : Real number or a list/matrix, the location (mean) or the
+        location vector
+    b : Real number or a positive definite matrix, representing a scale
+        or the covariance matrix.
 
     Returns
     =======
@@ -1563,7 +1624,7 @@ def Laplace(name, mu, b):
     ========
 
     >>> from sympy.stats import Laplace, density, cdf
-    >>> from sympy import Symbol
+    >>> from sympy import Symbol, pprint
 
     >>> mu = Symbol("mu")
     >>> b = Symbol("b", positive=True)
@@ -1578,6 +1639,13 @@ def Laplace(name, mu, b):
     Piecewise((exp((-mu + z)/b)/2, mu > z),
             (-exp((mu - z)/b)/2 + 1, True))
 
+    >>> L = Laplace('L', [1, 2], [[1, 0], [0, 1]])
+    >>> pprint(density(L)(1, 2), use_unicode=False)
+     5        /     ____\
+    e *besselk\0, \/ 35 /
+    ---------------------
+              pi
+
     References
     ==========
 
@@ -1585,6 +1653,12 @@ def Laplace(name, mu, b):
     .. [2] http://mathworld.wolfram.com/LaplaceDistribution.html
     """
 
+    if isinstance(mu, (list, MatrixBase)) and\
+        isinstance(b, (list, MatrixBase)):
+        from sympy.stats.joint_rv_types import MultivariateLaplaceDistribution
+        return multivariate_rv(
+            MultivariateLaplaceDistribution, name, mu, b)
+
     return rv(name, LaplaceDistribution, (mu, b))
 
 #-------------------------------------------------------------------------------
@@ -1605,6 +1679,9 @@ class LogisticDistribution(SingleContinuousDistribution):
     def _characteristic_function(self, t):
         return Piecewise((exp(I*t*self.mu) * pi*self.s*t / sinh(pi*self.s*t), Ne(t, 0)), (S.One, True))
 
+    def _moment_generating_function(self, t):
+        return exp(self.mu*t) * Beta(1 - self.s*t, 1 + self.s*t)
+
 def Logistic(name, mu, s):
     r"""
     Create a continuous random variable with a logistic distribution.
@@ -1675,6 +1752,8 @@ class LogNormalDistribution(SingleContinuousDistribution):
                 (S.Zero, True)
         )
 
+    def _moment_generating_function(self, t):
+        raise NotImplementedError('Moment generating function of the log-normal distribution is not defined.')
 
 def LogNormal(name, mean, std):
     r"""
@@ -1751,7 +1830,6 @@ class MaxwellDistribution(SingleContinuousDistribution):
         a = self.a
         return sqrt(2/pi)*x**2*exp(-x**2/(2*a**2))/a**3
 
-
 def Maxwell(name, a):
     r"""
     Create a continuous random variable with a Maxwell distribution.
@@ -1823,7 +1901,6 @@ class NakagamiDistribution(SingleContinuousDistribution):
                     (lowergamma(mu, (mu/omega)*x**2)/gamma(mu), x > 0),
                     (S.Zero, True))
 
-
 def Nakagami(name, mu, omega):
     r"""
     Create a continuous random variable with a Nakagami distribution.
@@ -1869,10 +1946,10 @@ def Nakagami(name, mu, omega):
     ----------------------------------
                 Gamma(mu)
 
-    >>> simplify(E(X, meijerg=True))
+    >>> simplify(E(X))
     sqrt(mu)*sqrt(omega)*gamma(mu + 1/2)/gamma(mu + 1)
 
-    >>> V = simplify(variance(X, meijerg=True))
+    >>> V = simplify(variance(X))
     >>> pprint(V, use_unicode=False)
                         2
              omega*Gamma (mu + 1/2)
@@ -1917,6 +1994,10 @@ class NormalDistribution(SingleContinuousDistribution):
         mean, std = self.mean, self.std
         return exp(I*mean*t - std**2*t**2/2)
 
+    def _moment_generating_function(self, t):
+        mean, std = self.mean, self.std
+        return exp(mean*t + std**2*t**2/2)
+
 def Normal(name, mean, std):
     r"""
     Create a continuous random variable with a Normal distribution.
@@ -1929,8 +2010,9 @@ def Normal(name, mean, std):
     Parameters
     ==========
 
-    mu : Real number, the mean
-    sigma : Real number, :math:`\sigma^2 > 0` the variance
+    mu : Real number or a list representing the mean or the mean vector
+    sigma : Real number or a positive definite sqaure matrix,
+         :math:`\sigma^2 > 0` the variance
 
     Returns
     =======
@@ -1946,7 +2028,7 @@ def Normal(name, mean, std):
     >>> mu = Symbol("mu")
     >>> sigma = Symbol("sigma", positive=True)
     >>> z = Symbol("z")
-
+    >>> y = Symbol("y")
     >>> X = Normal("x", mu, sigma)
 
     >>> density(X)(z)
@@ -1974,6 +2056,20 @@ def Normal(name, mean, std):
     >>> simplify(std(2*X + 1))
     2
 
+    >>> m = Normal('X', [1, 2], [[2, 1], [1, 2]])
+    >>> from sympy.stats.joint_rv import marginal_distribution
+    >>> pprint(density(m)(y, z))
+           /  y   1\ /2*y   z\   /  z    \ /  y   2*z    \
+           |- - + -|*|--- - -| + |- - + 1|*|- - + --- - 1|
+      ___  \  2   2/ \ 3    3/   \  2    / \  3    3     /
+    \/ 3 *e
+    ------------------------------------------------------
+                             6*pi
+
+    >>> marginal_distribution(m, m[0])(1)
+     1/(2*sqrt(pi))
+
+
     References
     ==========
 
@@ -1981,6 +2077,11 @@ def Normal(name, mean, std):
     .. [2] http://mathworld.wolfram.com/NormalDistributionFunction.html
     """
 
+    if isinstance(mean, (list, MatrixBase)) and\
+        isinstance(std, (list, MatrixBase)):
+        from sympy.stats.joint_rv_types import MultivariateNormalDistribution
+        return multivariate_rv(
+            MultivariateNormalDistribution, name, mean, std)
     return rv(name, NormalDistribution, (mean, std))
 
 #-------------------------------------------------------------------------------
@@ -2013,6 +2114,14 @@ class ParetoDistribution(SingleContinuousDistribution):
                 (0, True),
         )
 
+    def _moment_generating_function(self, t):
+        xm, alpha = self.xm, self.alpha
+        return alpha * (-xm*t)**alpha * uppergamma(-alpha, -xm*t)
+
+    def _characteristic_function(self, t):
+        xm, alpha = self.xm, self.alpha
+        return alpha * (-I * xm * t) ** alpha * uppergamma(-alpha, -I * xm * t)
+
 
 def Pareto(name, xm, alpha):
     r"""
@@ -2079,6 +2188,17 @@ class QuadraticUDistribution(SingleContinuousDistribution):
                   (alpha * (x-beta)**2, And(a<=x, x<=b)),
                   (S.Zero, True))
 
+    def _moment_generating_function(self, t):
+        a, b = self.a, self.b
+
+        return -3 * (exp(a*t) * (4  + (a**2 + 2*a*(-2 + b) + b**2) * t) - exp(b*t) * (4 + (-4*b + (a + b)**2) * t)) / ((a-b)**3 * t**2)
+
+    def _characteristic_function(self, t):
+        def _moment_generating_function(self, t):
+            a, b = self.a, self.b
+
+            return -3*I*(exp(I*a*t*exp(I*b*t)) * (4*I - (-4*b + (a+b)**2)*t)) / ((a-b)**3 * t**2)
+
 
 def QuadraticU(name, a, b):
     r"""
@@ -2161,6 +2281,9 @@ class RaisedCosineDistribution(SingleContinuousDistribution):
                          (exp(I*pi*mu/s)/2, Eq(t, pi/s)),
                          (pi**2*sin(s*t)*exp(I*mu*t) / (s*t*(pi**2 - s**2*t**2)), True))
 
+    def _moment_generating_function(self, t):
+        mu, s = self.mu, self.s
+        return pi**2 * sinh(s*t) * exp(mu*t) /  (s*t*(pi**2 + s**2*t**2))
 
 def RaisedCosine(name, mu, s):
     r"""
@@ -2231,6 +2354,10 @@ class RayleighDistribution(SingleContinuousDistribution):
         sigma = self.sigma
         return 1 - sigma*t*exp(-sigma**2*t**2/2) * sqrt(pi/2) * (erfi(sigma*t/sqrt(2)) - I)
 
+    def _moment_generating_function(self, t):
+        sigma = self.sigma
+        return 1 + sigma*t*exp(sigma**2*t**2/2) * sqrt(pi/2) * (erf(sigma*t/sqrt(2)) + 1)
+
 
 def Rayleigh(name, sigma):
     r"""
@@ -2300,7 +2427,6 @@ class ShiftedGompertzDistribution(SingleContinuousDistribution):
         b, eta = self.b, self.eta
         return b*exp(-b*x)*exp(-eta*exp(-b*x))*(1+eta*(1-exp(-b*x)))
 
-
 def ShiftedGompertz(name, b, eta):
     r"""
     Create a continuous random variable with a Shifted Gompertz distribution.
@@ -2361,6 +2487,8 @@ class StudentTDistribution(SingleContinuousDistribution):
         return S.Half + x*gamma((nu+1)/2)*hyper((S.Half, (nu+1)/2),
                                 (S(3)/2,), -x**2/nu)/(sqrt(pi*nu)*gamma(nu/2))
 
+    def _moment_generating_function(self, t):
+        raise NotImplementedError('The moment generating function for the Student-T distribution is undefined.')
 
 def StudentT(name, nu):
     r"""
@@ -2437,7 +2565,6 @@ class TrapezoidalDistribution(SingleContinuousDistribution):
             (2*(d-x) / ((d-c)*(d+c-a-b)), And(c <= x, x <= d)),
             (S.Zero, True))
 
-
 def Trapezoidal(name, a, b, c, d):
     r"""
     Create a continuous random variable with a trapezoidal distribution.
@@ -2517,6 +2644,15 @@ class TriangularDistribution(SingleContinuousDistribution):
             (2*(b - x)/((b - a)*(b - c)), And(c < x, x <= b)),
             (S.Zero, True))
 
+    def _characteristic_function(self, t):
+        a, b, c = self.a, self.b, self.c
+        return -2 *((b-c) * exp(I*a*t) - (b-a) * exp(I*c*t) + (c-a) * exp(I*b*t)) / ((b-a)*(c-a)*(b-c)*t**2)
+
+    def _moment_generating_function(self, t):
+        a, b, c = self.a, self.b, self.c
+        return 2 * ((b - c) * exp(a * t) - (b - a) * exp(c * t) + (c + a) * exp(b * t)) / (
+        (b - a) * (c - a) * (b - c) * t ** 2)
+
 
 def Triangular(name, a, b, c):
     r"""
@@ -2609,6 +2745,11 @@ class UniformDistribution(SingleContinuousDistribution):
         return Piecewise(((exp(I*t*right) - exp(I*t*left)) / (I*t*(right - left)), Ne(t, 0)),
                          (S.One, True))
 
+    def _moment_generating_function(self, t):
+        left, right = self.left, self.right
+        return Piecewise(((exp(t*right) - exp(t*left)) / (t * (right - left)), Ne(t, 0)),
+                         (S.One, True))
+
     def expectation(self, expr, var, **kwargs):
         from sympy import Max, Min
         kwargs['evaluate'] = True
@@ -2704,6 +2845,12 @@ class UniformSumDistribution(SingleContinuousDistribution):
                         (k, 0, floor(x))), x <= n),
                         (S.One, True))
 
+    def _characteristic_function(self, t):
+        return ((exp(I*t) - 1) / (I*t))**self.n
+
+    def _moment_generating_function(self, t):
+        return ((exp(t) - 1) / t)**self.n
+
 def UniformSum(name, n):
     r"""
     Create a continuous random variable with an Irwin-Hall distribution.
@@ -2789,7 +2936,6 @@ class VonMisesDistribution(SingleContinuousDistribution):
         mu, k = self.mu, self.k
         return exp(k*cos(x-mu)) / (2*pi*besseli(0, k))
 
-
 def VonMises(name, mu, k):
     r"""
     Create a Continuous Random Variable with a von Mises distribution.
@@ -2862,7 +3008,6 @@ class WeibullDistribution(SingleContinuousDistribution):
     def sample(self):
         return random.weibullvariate(self.alpha, self.beta)
 
-
 def Weibull(name, alpha, beta):
     r"""
     Create a continuous random variable with a Weibull distribution.
@@ -2937,6 +3082,10 @@ class WignerSemicircleDistribution(SingleContinuousDistribution):
         return Piecewise((2 * besselj(1, self.R*t) / (self.R*t), Ne(t, 0)),
                          (S.One, True))
 
+    def _moment_generating_function(self, t):
+        return Piecewise((2 * besseli(1, self.R*t) / (self.R*t), Ne(t, 0)),
+                         (S.One, True))
+
 def WignerSemicircle(name, R):
     r"""
     Create a continuous random variable with a Wigner semicircle distribution.
