@@ -1,6 +1,7 @@
 from __future__ import print_function, division
-import collections
-from sympy import Matrix, Integer, sympify, Basic, Derivative
+
+from sympy import Basic
+from sympy.core.compatibility import SYMPY_INTS, Iterable
 
 
 class NDimArray(object):
@@ -11,7 +12,7 @@ class NDimArray(object):
 
     Create an N-dim array of zeros:
 
-    >>> from sympy.tensor.array import MutableDenseNDimArray
+    >>> from sympy import MutableDenseNDimArray
     >>> a = MutableDenseNDimArray.zeros(2, 3, 4)
     >>> a
     [[[0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0]], [[0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0]]]
@@ -55,13 +56,16 @@ class NDimArray(object):
     [[-3, -3], [-3, -3]]
 
     """
-    def __new__(cls, *args, **kwargs):
+
+    _diff_wrt = True
+
+    def __new__(cls, iterable, shape=None, **kwargs):
         from sympy.tensor.array import ImmutableDenseNDimArray
-        return ImmutableDenseNDimArray(*args, **kwargs)
+        return ImmutableDenseNDimArray(iterable, shape, **kwargs)
 
     def _parse_index(self, index):
 
-        if isinstance(index, (int, Integer)):
+        if isinstance(index, (SYMPY_INTS, Integer)):
             if index >= self._loop_size:
                 raise ValueError("index out of range")
             return index
@@ -86,14 +90,26 @@ class NDimArray(object):
         index.reverse()
         return tuple(index)
 
+    def _check_symbolic_index(self, index):
+        # Check if any index is symbolic:
+        tuple_index = (index if isinstance(index, tuple) else (index,))
+        if any([(isinstance(i, Expr) and (not i.is_number)) for i in tuple_index]):
+            for i, nth_dim in zip(tuple_index, self.shape):
+                if ((i < 0) == True) or ((i >= nth_dim) == True):
+                    raise ValueError("index out of range")
+            from sympy.tensor import Indexed
+            return Indexed(self, *tuple_index)
+        return None
+
     def _setter_iterable_check(self, value):
-        if isinstance(value, (collections.Iterable, Matrix, NDimArray)):
+        from sympy.matrices.matrices import MatrixBase
+        if isinstance(value, (Iterable, MatrixBase, NDimArray)):
             raise NotImplementedError
 
     @classmethod
     def _scan_iterable_shape(cls, iterable):
         def f(pointer):
-            if not isinstance(pointer, collections.Iterable):
+            if not isinstance(pointer, Iterable):
                 return [pointer], ()
 
             result = []
@@ -108,6 +124,7 @@ class NDimArray(object):
 
     @classmethod
     def _handle_ndarray_creation_inputs(cls, iterable=None, shape=None, **kwargs):
+        from sympy.matrices.matrices import MatrixBase
 
         if shape is None and iterable is None:
             shape = ()
@@ -117,11 +134,11 @@ class NDimArray(object):
             shape = iterable.shape
             iterable = list(iterable)
         # Construct N-dim array from an iterable (numpy arrays included):
-        elif shape is None and isinstance(iterable, collections.Iterable):
+        elif shape is None and isinstance(iterable, Iterable):
             iterable, shape = cls._scan_iterable_shape(iterable)
 
         # Construct N-dim array from a Matrix:
-        elif shape is None and isinstance(iterable, Matrix):
+        elif shape is None and isinstance(iterable, MatrixBase):
             shape = iterable.shape
 
         # Construct N-dim array from another N-dim array:
@@ -133,12 +150,13 @@ class NDimArray(object):
             pass
 
         else:
-            raise TypeError("Data type not understood")
+            shape = ()
+            iterable = (iterable,)
 
-        if isinstance(shape, (int, Integer)):
+        if isinstance(shape, (SYMPY_INTS, Integer)):
             shape = (shape,)
 
-        if any([not isinstance(dim, (int, Integer)) for dim in shape]):
+        if any([not isinstance(dim, (SYMPY_INTS, Integer)) for dim in shape]):
             raise TypeError("Shape should contain integers only.")
 
         return tuple(shape), iterable
@@ -149,7 +167,7 @@ class NDimArray(object):
         Examples
         ========
 
-        >>> from sympy.tensor.array.dense_ndim_array import MutableDenseNDimArray
+        >>> from sympy import MutableDenseNDimArray
         >>> a = MutableDenseNDimArray.zeros(3, 3)
         >>> a
         [[0, 0, 0], [0, 0, 0], [0, 0, 0]]
@@ -167,7 +185,7 @@ class NDimArray(object):
         Examples
         ========
 
-        >>> from sympy.tensor.array.dense_ndim_array import MutableDenseNDimArray
+        >>> from sympy import MutableDenseNDimArray
         >>> a = MutableDenseNDimArray.zeros(3, 3)
         >>> a.shape
         (3, 3)
@@ -182,7 +200,7 @@ class NDimArray(object):
         Examples
         ========
 
-        >>> from sympy.tensor.array.dense_ndim_array import MutableDenseNDimArray
+        >>> from sympy import MutableDenseNDimArray
         >>> a = MutableDenseNDimArray.zeros(3,4,5,6,3)
         >>> a.rank()
         5
@@ -197,14 +215,39 @@ class NDimArray(object):
         Examples
         ========
 
-        >>> from sympy.tensor.array import ImmutableDenseNDimArray
+        >>> from sympy import ImmutableDenseNDimArray
         >>> from sympy.abc import x, y
         >>> M = ImmutableDenseNDimArray([[x, y], [1, x*y]])
         >>> M.diff(x)
         [[1, 0], [0, y]]
 
         """
-        return type(self)(map(lambda x: x.diff(*args), self), self.shape)
+        from sympy import Derivative
+        return Derivative(self.as_immutable(), *args, evaluate=True)
+
+    def _accept_eval_derivative(self, s):
+        return s._visit_eval_derivative_array(self)
+
+    def _visit_eval_derivative_scalar(self, base):
+        # Types are (base: scalar, self: array)
+        return self.applyfunc(lambda x: base.diff(x))
+
+    def _visit_eval_derivative_array(self, base):
+        # Types are (base: array/matrix, self: array)
+        from sympy import derive_by_array
+        return derive_by_array(base, self)
+
+    def _eval_derivative_n_times(self, s, n):
+        return Basic._eval_derivative_n_times(self, s, n)
+
+    def _eval_derivative(self, arg):
+        from sympy import derive_by_array
+        from sympy import Derivative, Tuple
+        from sympy.matrices.common import MatrixCommon
+        if isinstance(arg, (Iterable, Tuple, MatrixCommon, NDimArray)):
+            return derive_by_array(self, arg)
+        else:
+            return self.applyfunc(lambda x: x.diff(arg))
 
     def applyfunc(self, f):
         """Apply a function to each element of the N-dim array.
@@ -212,7 +255,7 @@ class NDimArray(object):
         Examples
         ========
 
-        >>> from sympy.tensor.array import ImmutableDenseNDimArray
+        >>> from sympy import ImmutableDenseNDimArray
         >>> m = ImmutableDenseNDimArray([i*2+j for i in range(2) for j in range(2)], (2, 2))
         >>> m
         [[0, 1], [2, 3]]
@@ -227,7 +270,7 @@ class NDimArray(object):
         Examples
         ========
 
-        >>> from sympy.tensor.array import MutableDenseNDimArray
+        >>> from sympy import MutableDenseNDimArray
         >>> a = MutableDenseNDimArray.zeros(2, 2)
         >>> a
         [[0, 0], [0, 0]]
@@ -240,33 +283,22 @@ class NDimArray(object):
             sh //= shape_left[0]
             return "[" + ", ".join([f(sh, shape_left[1:], i+e*sh, i+(e+1)*sh) for e in range(shape_left[0])]) + "]" # + "\n"*len(shape_left)
 
+        if self.rank() == 0:
+            return self[()].__str__()
+
         return f(self._loop_size, self.shape, 0, self._loop_size)
 
-        out_str = ''
-
-        # forming output string
-        for i, el in enumerate(self):
-
-            out_str += str(el) + '  '
-            chidx = i+1
-            for sh in reversed(self.shape):
-                if chidx % sh == 0:
-                    out_str += '\n'
-                    chidx //= sh
-
-        return out_str
-
     def __repr__(self):
         return self.__str__()
 
     def tolist(self):
         """
-        Conveting MutableDenseNDimArray to one-dim list
+        Converting MutableDenseNDimArray to one-dim list
 
         Examples
         ========
 
-        >>> from sympy.tensor.array import MutableDenseNDimArray
+        >>> from sympy import MutableDenseNDimArray
         >>> a = MutableDenseNDimArray([1, 2, 3, 4], (2, 2))
         >>> a
         [[1, 2], [3, 4]]
@@ -307,21 +339,27 @@ class NDimArray(object):
         return type(self)(result_list, self.shape)
 
     def __mul__(self, other):
-        if isinstance(other, (collections.Iterable,NDimArray, Matrix)):
+        from sympy.matrices.matrices import MatrixBase
+
+        if isinstance(other, (Iterable, NDimArray, MatrixBase)):
             raise ValueError("scalar expected, use tensorproduct(...) for tensorial product")
         other = sympify(other)
         result_list = [i*other for i in self]
         return type(self)(result_list, self.shape)
 
     def __rmul__(self, other):
-        if isinstance(other, (collections.Iterable,NDimArray, Matrix)):
+        from sympy.matrices.matrices import MatrixBase
+
+        if isinstance(other, (Iterable, NDimArray, MatrixBase)):
             raise ValueError("scalar expected, use tensorproduct(...) for tensorial product")
         other = sympify(other)
         result_list = [other*i for i in self]
         return type(self)(result_list, self.shape)
 
     def __div__(self, other):
-        if isinstance(other, (collections.Iterable,NDimArray, Matrix)):
+        from sympy.matrices.matrices import MatrixBase
+
+        if isinstance(other, (Iterable, NDimArray, MatrixBase)):
             raise ValueError("scalar expected")
         other = sympify(other)
         result_list = [i/other for i in self]
@@ -330,6 +368,10 @@ class NDimArray(object):
     def __rdiv__(self, other):
         raise NotImplementedError('unsupported operation on NDimArray')
 
+    def __neg__(self):
+        result_list = [-i for i in self]
+        return type(self)(result_list, self.shape)
+
     def __eq__(self, other):
         """
         NDimArray instances can be compared to each other.
@@ -338,7 +380,7 @@ class NDimArray(object):
         Examples
         ========
 
-        >>> from sympy.tensor.array import MutableDenseNDimArray
+        >>> from sympy import MutableDenseNDimArray
         >>> a = MutableDenseNDimArray.zeros(2, 3)
         >>> b = MutableDenseNDimArray.zeros(2, 3)
         >>> a == b
@@ -356,17 +398,47 @@ class NDimArray(object):
         return (self.shape == other.shape) and (list(self) == list(other))
 
     def __ne__(self, other):
-        return not self.__eq__(other)
+        return not self == other
 
     __truediv__ = __div__
     __rtruediv__ = __rdiv__
 
-    def _eval_diff(self, *args, **kwargs):
-        if kwargs.pop("evaluate", True):
-            return self.diff(*args)
-        else:
-            return Derivative(self, *args, **kwargs)
+    def _eval_transpose(self):
+        if self.rank() != 2:
+            raise ValueError("array rank not 2")
+        from .arrayop import permutedims
+        return permutedims(self, (1, 0))
+
+    def transpose(self):
+        return self._eval_transpose()
+
+    def _eval_conjugate(self):
+        return self.func([i.conjugate() for i in self], self.shape)
+
+    def conjugate(self):
+        return self._eval_conjugate()
+
+    def _eval_adjoint(self):
+        return self.transpose().conjugate()
+
+    def adjoint(self):
+        return self._eval_adjoint()
 
 
 class ImmutableNDimArray(NDimArray, Basic):
     _op_priority = 11.0
+
+    def __hash__(self):
+        return Basic.__hash__(self)
+
+    def as_immutable(self):
+        return self
+
+    def as_mutable(self):
+        raise NotImplementedError("abstract method")
+
+
+from sympy.core.numbers import Integer
+from sympy.core.sympify import sympify
+from sympy.core.function import Derivative
+from sympy.core.expr import Expr
