@@ -5,7 +5,7 @@ import itertools
 
 from sympy.core.sympify import _sympify
 
-from sympy import S, Dict, flatten, SparseMatrix, Basic, Tuple
+from sympy import S, Dict, Basic, Tuple
 from sympy.tensor.array.mutable_ndim_array import MutableNDimArray
 from sympy.tensor.array.ndim_array import NDimArray, ImmutableNDimArray
 
@@ -22,7 +22,7 @@ class SparseNDimArray(NDimArray):
         Examples
         ========
 
-        >>> from sympy.tensor.array import MutableSparseNDimArray
+        >>> from sympy import MutableSparseNDimArray
         >>> a = MutableSparseNDimArray(range(4), (2, 2))
         >>> a
         [[0, 1], [2, 3]]
@@ -35,7 +35,22 @@ class SparseNDimArray(NDimArray):
         >>> a[2]
         2
 
+        Symbolic indexing:
+
+        >>> from sympy.abc import i, j
+        >>> a[i, j]
+        [[0, 1], [2, 3]][i, j]
+
+        Replace `i` and `j` to get element `(0, 0)`:
+
+        >>> a[i, j].subs({i: 0, j: 0})
+        0
+
         """
+        syindex = self._check_symbolic_index(index)
+        if syindex is not None:
+            return syindex
+
         # `index` is a tuple with one or more slices:
         if isinstance(index, tuple) and any([isinstance(i, slice) for i in index]):
 
@@ -75,7 +90,7 @@ class SparseNDimArray(NDimArray):
         Examples
         ========
 
-        >>> from sympy.tensor.array import MutableSparseNDimArray
+        >>> from sympy import MutableSparseNDimArray
         >>> a = MutableSparseNDimArray([1 for i in range(9)], (3, 3))
         >>> b = a.tomatrix()
         >>> b
@@ -84,6 +99,7 @@ class SparseNDimArray(NDimArray):
         [1, 1, 1],
         [1, 1, 1]])
         """
+        from sympy.matrices import SparseMatrix
         if self.rank() != 2:
             raise ValueError('Dimensions must be of size of 2')
 
@@ -109,9 +125,10 @@ class SparseNDimArray(NDimArray):
 
 class ImmutableSparseNDimArray(SparseNDimArray, ImmutableNDimArray):
 
-    def __new__(cls, *args, **kwargs):
+    def __new__(cls, iterable=None, shape=None, **kwargs):
+        from sympy.utilities.iterables import flatten
 
-        shape, flat_list = cls._handle_ndarray_creation_inputs(*args, **kwargs)
+        shape, flat_list = cls._handle_ndarray_creation_inputs(iterable, shape, **kwargs)
         shape = Tuple(*map(_sympify, shape))
         loop_size = functools.reduce(lambda x,y: x*y, shape) if shape else 0
 
@@ -137,12 +154,16 @@ class ImmutableSparseNDimArray(SparseNDimArray, ImmutableNDimArray):
     def __setitem__(self, index, value):
         raise TypeError("immutable N-dim array")
 
+    def as_mutable(self):
+        return MutableSparseNDimArray(self)
+
 
 class MutableSparseNDimArray(MutableNDimArray, SparseNDimArray):
 
-    def __new__(cls, *args, **kwargs):
+    def __new__(cls, iterable=None, shape=None, **kwargs):
+        from sympy.utilities.iterables import flatten
 
-        shape, flat_list = cls._handle_ndarray_creation_inputs(*args, **kwargs)
+        shape, flat_list = cls._handle_ndarray_creation_inputs(iterable, shape, **kwargs)
         self = object.__new__(cls)
         self._shape = shape
         self._rank = len(shape)
@@ -167,7 +188,7 @@ class MutableSparseNDimArray(MutableNDimArray, SparseNDimArray):
         Examples
         ========
 
-        >>> from sympy.tensor.array import MutableSparseNDimArray
+        >>> from sympy import MutableSparseNDimArray
         >>> a = MutableSparseNDimArray.zeros(2, 2)
         >>> a[0, 0] = 1
         >>> a[1, 1] = 1
@@ -187,3 +208,10 @@ class MutableSparseNDimArray(MutableNDimArray, SparseNDimArray):
             self._sparse_array.pop(index)
         else:
             self._sparse_array[index] = value
+
+    def as_immutable(self):
+        return ImmutableSparseNDimArray(self)
+
+    @property
+    def free_symbols(self):
+        return {i for j in self._sparse_array.values() for i in j.free_symbols}
